@@ -1,3 +1,4 @@
+/* SPDX-License-Identifier: GPL-2.0-only */
 #ifndef DRBD_TRANSPORT_H
 #define DRBD_TRANSPORT_H
 
@@ -11,7 +12,7 @@
    So that transport compiled against an older version of this
    header will no longer load in a module that assumes a newer
    version. */
-#define DRBD_TRANSPORT_API_VERSION 18
+#define DRBD_TRANSPORT_API_VERSION 20
 
 /* MSG_MSG_DONTROUTE and MSG_PROBE are not used by DRBD. I.e.
    we can reuse these flags for our purposes */
@@ -54,8 +55,8 @@
 	} while (0)
 
 struct drbd_resource;
-struct drbd_connection;
-struct drbd_peer_device;
+struct drbd_listener;
+struct drbd_transport;
 
 enum drbd_stream {
 	DATA_STREAM,
@@ -85,8 +86,6 @@ enum drbd_tr_event {
 	TIMEOUT,
 };
 
-struct drbd_listener;
-
 /* A transport might wrap its own data structure around this. Having
    this base class as its first member. */
 struct drbd_path {
@@ -95,10 +94,12 @@ struct drbd_path {
 
 	struct kref kref;
 
+	struct net *net;
 	int my_addr_len;
 	int peer_addr_len;
 	bool established; /* updated by the transport */
 
+	struct drbd_transport *transport;
 	struct list_head list; /* paths of a connection */
 	struct list_head listener_link; /* paths waiting for an incomming connection,
 					   head is in a drbd_listener */
@@ -110,13 +111,12 @@ struct drbd_path {
 /* Each transport implementation should embed a struct drbd_transport
    into it's instance data structure. */
 struct drbd_transport {
-	struct drbd_transport_ops *ops;
 	struct drbd_transport_class *class;
 
 	struct list_head paths;
 
 	const char *log_prefix;		/* resource name */
-	struct net_conf *net_conf;	/* content protected by rcu */
+	struct net_conf __rcu *net_conf;	/* content protected by rcu */
 
 	/* These members are intended to be updated by the transport: */
 	unsigned int ko_count;
@@ -142,7 +142,11 @@ struct drbd_const_buffer {
 };
 
 struct drbd_transport_ops {
+	int (*init)(struct drbd_transport *);
 	void (*free)(struct drbd_transport *, enum drbd_tr_free_op free_op);
+	int (*init_listener)(struct drbd_transport *, const struct sockaddr *, struct net *net,
+			     struct drbd_listener *);
+	void (*release_listener)(struct drbd_listener *);
 	int (*connect)(struct drbd_transport *);
 
 /**
@@ -193,7 +197,20 @@ struct drbd_transport_ops {
 	int (*recv_pages)(struct drbd_transport *, struct drbd_page_chain_head *, size_t size);
 
 	void (*stats)(struct drbd_transport *, struct drbd_transport_stats *stats);
-	void (*net_conf_change)(struct drbd_transport *, struct net_conf *new_net_conf);
+/**
+ * net_conf_change() - Notify about changed network configuration on the transport.
+ * @new_net_conf: The new network configuration that should be applied.
+ *
+ * net_conf_change() is called in the context of either the initial creation of the connection,
+ * or when the net_conf is changed via netlink. Note that assignment of the net_conf to the
+ * transport object happens after this function is called.
+ *
+ * On a negative (error) return value, it is expected that any changes are reverted and
+ * the old net_conf (if any) is still in effect.
+ *
+ * Upon success the function return 0. Upon error the function returns a negative value.
+ */
+	int (*net_conf_change)(struct drbd_transport *, struct net_conf *new_net_conf);
 	void (*set_rcvtimeo)(struct drbd_transport *, enum drbd_stream, long timeout);
 	long (*get_rcvtimeo)(struct drbd_transport *, enum drbd_stream);
 	int (*send_page)(struct drbd_transport *, enum drbd_stream, struct page *,
@@ -202,8 +219,8 @@ struct drbd_transport_ops {
 	bool (*stream_ok)(struct drbd_transport *, enum drbd_stream);
 	bool (*hint)(struct drbd_transport *, enum drbd_stream, enum drbd_tr_hints hint);
 	void (*debugfs_show)(struct drbd_transport *, struct seq_file *m);
-	int (*add_path)(struct drbd_transport *, struct drbd_path *path);
-	int (*remove_path)(struct drbd_transport *, struct drbd_path *path);
+	int (*add_path)(struct drbd_path *path);
+	int (*remove_path)(struct drbd_path *path);
 };
 
 struct drbd_transport_class {
@@ -211,8 +228,10 @@ struct drbd_transport_class {
 	const int instance_size;
 	const int path_instance_size;
 	const int listener_instance_size;
+	struct drbd_transport_ops ops;
+
 	struct module *module;
-	int (*init)(struct drbd_transport *);
+
 	struct list_head list;
 };
 
@@ -223,6 +242,7 @@ struct drbd_transport_class {
 struct drbd_listener {
 	struct kref kref;
 	struct drbd_resource *resource;
+	struct drbd_transport_class *transport_class;
 	struct list_head list; /* link for resource->listeners */
 	struct list_head waiters; /* list head for paths */
 	spinlock_t waiters_lock;
@@ -230,7 +250,6 @@ struct drbd_listener {
 	struct sockaddr_storage listen_addr;
 	struct completion ready;
 	int err;
-	void (*destroy)(struct drbd_listener *);
 };
 
 /* drbd_main.c */
@@ -245,13 +264,13 @@ extern struct drbd_transport_class *drbd_get_transport_class(const char *transpo
 extern void drbd_put_transport_class(struct drbd_transport_class *);
 extern void drbd_print_transports_loaded(struct seq_file *seq);
 
-extern int drbd_get_listener(struct drbd_transport *transport, struct drbd_path *path,
-			     int (*init_fn)(struct drbd_transport *, const struct sockaddr *, struct drbd_listener *));
+extern int drbd_get_listener(struct drbd_path *path);
 extern void drbd_put_listener(struct drbd_path *path);
 extern struct drbd_path *drbd_find_path_by_addr(struct drbd_listener *, struct sockaddr_storage *);
 extern bool drbd_stream_send_timed_out(struct drbd_transport *transport, enum drbd_stream stream);
 extern bool drbd_should_abort_listening(struct drbd_transport *transport);
 extern void drbd_path_event(struct drbd_transport *transport, struct drbd_path *path, bool destroyed);
+extern void drbd_listener_destroy(struct kref *kref);
 
 /* drbd_receiver.c*/
 extern struct page *drbd_alloc_pages(struct drbd_transport *, unsigned int, gfp_t);
