@@ -60,6 +60,7 @@ const char *drbd_buildtag(void);
 #define E_SYNTAX	  2
 #define E_CONFIG_INVALID 10
 #define E_EXEC_ERROR     20
+#define E_NO_MEM         30
 #define E_THINKO	 42 /* :) */
 
 enum {
@@ -133,5 +134,6 @@ static inline int m_system_ex(char **argv, int flags, const char *res_name)
 	m__system(argv, flags, res_name, NULL, NULL, &ex);
 	return ex;
 }
+extern int inject_cmd_failure(void);
 
 #endif
