@@ -219,6 +219,7 @@ static bool adjust_paths(const struct cfg_ctx *ctx, struct connection *running_c
 		running_path = find_path_by_addrs(running_conn, configured_path);
 		if (!running_path) {
 			tmp_ctx.path = configured_path;
+			configured_path->adj_new = true;
 			schedule_deferred_cmd(&new_path_cmd, &tmp_ctx, CFG_NET_PATH);
 		} else {
 			running_path->adj_seen = 1;
@@ -229,7 +230,8 @@ static bool adjust_paths(const struct cfg_ctx *ctx, struct connection *running_c
 		nr_running++;
 		if (!running_path->adj_seen) {
 			tmp_ctx.path = running_path;
-			schedule_deferred_cmd(&del_path_cmd, &tmp_ctx, CFG_NET_PREP_DOWN);
+			schedule_deferred_cmd(&del_path_cmd, &tmp_ctx,
+					      CFG_NET_PREP_DOWN | RETRY_AFTER_CONNECT);
 			del_path = true;
 		}
 	}
@@ -390,7 +392,7 @@ static int proxy_reconf(const struct cfg_ctx *ctx, struct connection *running_co
 		goto redo_whole_conn;
 
 	running_path = STAILQ_FIRST(&running_conn->paths); /* multiple paths via proxy, later! */
-	if (!running_path->my_proxy)
+	if (!running_path || !running_path->my_proxy)
 		goto redo_whole_conn;
 
 	if (running_path->proxy_conn_is_down)
@@ -737,7 +739,8 @@ adjust_net(const struct cfg_ctx *ctx, struct d_resource* running)
 			configured_conn = matching_conn(conn, &ctx->res->connections, true);
 			if (!configured_conn) {
 				struct cfg_ctx tmp_ctx = { .res = running, .conn = conn };
-				schedule_deferred_cmd(&del_peer_cmd, &tmp_ctx, CFG_NET_PREP_DOWN);
+				schedule_deferred_cmd(&del_peer_cmd, &tmp_ctx,
+						      CFG_NET_PREP_DOWN | RETRY_AFTER_CONNECT);
 			}
 		}
 	}
@@ -753,6 +756,7 @@ adjust_net(const struct cfg_ctx *ctx, struct d_resource* running)
 		if (running)
 			running_conn = matching_conn(conn, &running->connections, false);
 		if (!running_conn) {
+			conn->adj_new = true; /* making it as new for the retry case */
 			schedule_deferred_cmd(&new_peer_cmd, &tmp_ctx, CFG_NET_PREP_UP);
 			schedule_deferred_cmd(&new_path_cmd, &tmp_ctx, CFG_NET_PATH);
 			schedule_deferred_cmd(&connect_cmd, &tmp_ctx, CFG_NET_CONNECT);
