@@ -64,7 +64,7 @@ struct d_name *names_from_str(char* str)
 {
 	struct d_name *names;
 
-	names = malloc(sizeof(struct d_name));
+	names = checked_malloc(sizeof(struct d_name));
 	names->name = strdup(str);
 
 	return names;
@@ -204,7 +204,7 @@ void range_check(const enum range_checks what, const char *name,
 
 struct d_option *new_opt(char *name, char *value)
 {
-	struct d_option *cn = calloc(1, sizeof(struct d_option));
+	struct d_option *cn = checked_calloc(1, sizeof(struct d_option));
 
 	/* log_err("%s:%d: %s = %s\n",config_file,line,name,value); */
 	cn->name = name;
@@ -300,12 +300,7 @@ int vcheck_uniq_file_line(
 	if (config_valid >= 2)
 		return 1;
 
-	e = calloc(1, sizeof *e);
-	if (!e) {
-		log_err("calloc: %m\n");
-		exit(E_THINKO);
-	}
-
+	e = checked_calloc(1, sizeof *e);
 	rv = vasprintf(&e->key, fmt, ap);
 
 	if (rv < 0) {
@@ -637,23 +632,24 @@ void parse_options_syncer(struct d_resource *res)
 	}
 }
 
-static struct options __parse_options(struct context_def *options_def,
+static void __parse_options(struct options *options,
+				      struct context_def *options_def,
 				      void (*delegate)(void*),
 				      void *delegate_context)
 {
-	struct options options = STAILQ_HEAD_INITIALIZER(options);
 	const struct field_def *field_def;
 	char *value;
 	bool no_prefix;
 	int token;
 
+	STAILQ_INIT(options);
 	c_section_start = line;
 	fline = line;
 
 	while (1) {
 		token = yylex();
 		if (token == '}')
-			return options;
+			return;
 
 		field_def = find_field(&no_prefix, options_def, yytext);
 		if (!field_def) {
@@ -666,13 +662,13 @@ static struct options __parse_options(struct context_def *options_def,
 		}
 
 		value = parse_option_value(field_def, no_prefix);
-		insert_tail(&options, new_opt((char *)field_def->name, value));
+		insert_tail(options, new_opt((char *)field_def->name, value));
 	}
 }
 
-static struct options parse_options(struct context_def *options_def)
+static void parse_options(struct options *options, struct context_def *options_def)
 {
-	return __parse_options(options_def, NULL, NULL);
+	__parse_options(options, options_def, NULL, NULL);
 }
 
 static void insert_pd_options_delegate(void *ctx)
@@ -703,9 +699,7 @@ static void parse_disk_options(struct options *disk_options,
 {
 	struct peer_delegate_data params = {device_options, peer_device_options};
 
-	*disk_options = __parse_options(&attach_cmd_ctx,
-					&insert_pd_options_delegate,
-					&params);
+	__parse_options(disk_options, &attach_cmd_ctx, &insert_pd_options_delegate, &params);
 }
 
 static void __parse_address(struct d_address *a)
@@ -767,7 +761,7 @@ static void parse_hosts(struct names *hosts, char delimeter)
 		token = yylex();
 		switch (token) {
 		case TK_STRING:
-			name = malloc(sizeof(struct d_name));
+			name = checked_malloc(sizeof(struct d_name));
 			name->name = yylval.txt;
 			insert_tail(hosts, name);
 			nr_hosts++;
@@ -789,7 +783,7 @@ static struct d_proxy_info *parse_proxy_section(void)
 {
 	struct d_proxy_info *proxy;
 
-	proxy = calloc(1, sizeof(struct d_proxy_info));
+	proxy = checked_calloc(1, sizeof(struct d_proxy_info));
 	STAILQ_INIT(&proxy->on_hosts);
 
 	EXP(TK_ON);
@@ -887,11 +881,7 @@ struct d_volume *alloc_volume(void)
 {
 	struct d_volume *vol;
 
-	vol = calloc(1, sizeof(struct d_volume));
-	if (vol == NULL) {
-		log_err("calloc: %m\n");
-		exit(E_EXEC_ERROR);
-	}
+	vol = checked_calloc(1, sizeof(struct d_volume));
 
 	STAILQ_INIT(&vol->device_options);
 	STAILQ_INIT(&vol->disk_options);
@@ -1067,7 +1057,7 @@ static void parse_host_section(struct d_resource *res,
 	c_section_start = line;
 	fline = line;
 
-	host = calloc(1,sizeof(struct d_host_info));
+	host = checked_calloc(1,sizeof(struct d_host_info));
 	STAILQ_INIT(&host->res_options);
 	STAILQ_INIT(&host->volumes);
 	host->on_hosts = *on_hosts;
@@ -1159,7 +1149,7 @@ static void parse_host_section(struct d_resource *res,
 			break;
 		case TK_OPTIONS:
 			EXP('{');
-			host->res_options = parse_options(&resource_options_ctx);
+			parse_options(&host->res_options, &resource_options_ctx);
 			break;
 		case TK_SKIP:
 			parse_skip();
@@ -1210,7 +1200,7 @@ void parse_stacked_section(struct d_resource* res)
 	c_section_start = line;
 	fline = line;
 
-	host = calloc(1, sizeof(struct d_host_info));
+	host = checked_calloc(1, sizeof(struct d_host_info));
 	STAILQ_INIT(&host->res_options);
 	STAILQ_INIT(&host->on_hosts);
 	STAILQ_INIT(&host->volumes);
@@ -1333,16 +1323,12 @@ void proxy_delegate(void *ctx)
 				exit(E_CONFIG_INVALID);
 			}
 
-			word = malloc(sizeof(struct d_name));
-			if (!word)
-				pdperror("out of memory.");
+			word = checked_malloc(sizeof(struct d_name));
 			word->name = yylval.txt ? yylval.txt : strdup(yytext);
 			insert_tail(&line, word);
 		}
 
-		opt = calloc(1, sizeof(struct d_option));
-		if (!opt)
-			pdperror("out of memory.");
+		opt = checked_calloc(1, sizeof(struct d_option));
 		opt->name = strdup(names_to_str(&line));
 		insert_tail(&options, opt);
 		free_names(&line);
@@ -1357,8 +1343,7 @@ static int parse_proxy_options(struct options *proxy_options, struct options *pr
 	struct options opts;
 
 	EXP('{');
-	opts = __parse_options(&proxy_options_ctx,
-			       proxy_delegate, proxy_plugins);
+	__parse_options(&opts, &proxy_options_ctx, proxy_delegate, proxy_plugins);
 
 	if (proxy_options)
 		*proxy_options = opts;
@@ -1372,7 +1357,7 @@ int parse_proxy_options_section(struct d_proxy_info **pp)
 	struct d_proxy_info *proxy;
 
 	if (!*pp)
-		*pp = calloc(1, sizeof(struct d_proxy_info));
+		*pp = checked_calloc(1, sizeof(struct d_proxy_info));
 	proxy = *pp;
 
 	token = yylex();
@@ -1408,7 +1393,7 @@ static struct hname_address *parse_hname_address_pair(struct path *path, int pre
 	struct hname_address *ha;
 	int token;
 
-	ha = calloc(1, sizeof(struct hname_address));
+	ha = checked_calloc(1, sizeof(struct hname_address));
 	ha->config_line = line;
 
 	switch (prev_token) {
@@ -1466,11 +1451,7 @@ struct connection *alloc_connection()
 {
 	struct connection *conn;
 
-	conn = calloc(1, sizeof(struct connection));
-	if (conn == NULL) {
-		log_err("calloc: %m\n");
-		exit(E_EXEC_ERROR);
-	}
+	conn = checked_calloc(1, sizeof(struct connection));
 	STAILQ_INIT(&conn->paths);
 	STAILQ_INIT(&conn->net_options);
 	STAILQ_INIT(&conn->peer_devices);
@@ -1488,11 +1469,7 @@ struct peer_device *alloc_peer_device()
 {
 	struct peer_device *peer_device;
 
-	peer_device = calloc(1, sizeof(*peer_device));
-	if (!peer_device)  {
-		log_err("calloc: %m\n");
-		exit(E_EXEC_ERROR);
-	}
+	peer_device = checked_calloc(1, sizeof(*peer_device));
 	STAILQ_INIT(&peer_device->pd_options);
 
 	return peer_device;
@@ -1509,7 +1486,7 @@ static struct peer_device *parse_peer_device(int vnr)
 	EXP('{');
 	EXP(TK_DISK);
 	EXP('{');
-	peer_device->pd_options = parse_options(&peer_device_options_ctx);
+	parse_options(&peer_device->pd_options, &peer_device_options_ctx);
 	EXP('}');
 
 	return peer_device;
@@ -1519,7 +1496,7 @@ static struct d_host_info *parse_peer_node_id(void)
 {
 	struct d_host_info *host;
 
-	host = calloc(1,sizeof(struct d_host_info));
+	host = checked_calloc(1,sizeof(struct d_host_info));
 	STAILQ_INIT(&host->res_options);
 	STAILQ_INIT(&host->volumes);
 	STAILQ_INIT(&host->on_hosts);
@@ -1540,11 +1517,7 @@ struct path *alloc_path()
 {
 	struct path *path;
 
-	path = calloc(1, sizeof(struct path));
-	if (path == NULL) {
-		log_err("calloc: %m\n");
-		exit(E_EXEC_ERROR);
-	}
+	path = checked_calloc(1, sizeof(struct path));
 	STAILQ_INIT(&path->hname_address_pairs);
 
 	return path;
@@ -1648,15 +1621,15 @@ static struct connection *parse_connection(enum pr_flags flags)
 				config_valid = 0;
 			}
 			EXP('{');
-			conn->net_options = __parse_options(&show_net_options_ctx,
-							    &net_delegate, (void *)flags);
+			__parse_options(&conn->net_options, &show_net_options_ctx,
+					&net_delegate, (void *)flags);
 			break;
 		case TK_SKIP:
 			parse_skip();
 			break;
 		case TK_DISK:
 			EXP('{');
-			conn->pd_options = parse_options(&peer_device_options_ctx);
+			parse_options(&conn->pd_options, &peer_device_options_ctx);
 			break;
 		case TK_VOLUME:
 			EXP(TK_INTEGER);
@@ -1690,7 +1663,7 @@ void parse_connection_mesh(struct d_resource *res, enum pr_flags flags)
 	int token;
 
 	EXP('{');
-	mesh = calloc(1, sizeof(struct mesh));
+	mesh = checked_calloc(1, sizeof(struct mesh));
 	STAILQ_INIT(&mesh->hosts);
 	STAILQ_INIT(&mesh->net_options);
 
@@ -1707,10 +1680,8 @@ void parse_connection_mesh(struct d_resource *res, enum pr_flags flags)
 				config_valid = 0;
 			}
 			EXP('{');
-			mesh->net_options =
-				__parse_options(&show_net_options_ctx,
-						&net_delegate,
-						(void *)flags);
+			__parse_options(&mesh->net_options, &show_net_options_ctx,
+					&net_delegate, (void *)flags);
 			break;
 		case '}':
 			insert_tail(&res->meshes, mesh);
@@ -1733,7 +1704,7 @@ struct d_resource* parse_resource(char* res_name, enum pr_flags flags)
 	check_upr_init();
 	check_uniq("resource section", res_name);
 
-	res = calloc(1, sizeof(struct d_resource));
+	res = checked_calloc(1, sizeof(struct d_resource));
 	STAILQ_INIT(&res->volumes);
 	STAILQ_INIT(&res->connections);
 	STAILQ_INIT(&res->all_hosts);
@@ -1804,8 +1775,8 @@ struct d_resource* parse_resource(char* res_name, enum pr_flags flags)
 		case TK_NET:
 			check_upr("net section", "%s:net", res->name);
 			EXP('{');
-			options = __parse_options(&show_net_options_ctx,
-						  &net_delegate, (void *)flags);
+			__parse_options(&options, &show_net_options_ctx,
+					net_delegate, (void *)flags);
 
 			STAILQ_CONCAT(&res->net_options, &options);
 			break;
@@ -1817,14 +1788,14 @@ struct d_resource* parse_resource(char* res_name, enum pr_flags flags)
 		case TK_STARTUP:
 			check_upr("startup section", "%s:startup", res->name);
 			EXP('{');
-			res->startup_options = __parse_options(&startup_options_ctx,
-							       &startup_delegate,
-							       res);
+			__parse_options(&res->startup_options, &startup_options_ctx,
+					&startup_delegate,
+					res);
 			break;
 		case TK_HANDLER:
 			check_upr("handlers section", "%s:handlers", res->name);
 			EXP('{');
-			res->handlers = parse_options(&handlers_ctx);
+			parse_options(&res->handlers, &handlers_ctx);
 			break;
 		case TK_PROXY:
 			check_upr("proxy section", "%s:proxy", res->name);
@@ -1843,7 +1814,7 @@ struct d_resource* parse_resource(char* res_name, enum pr_flags flags)
 		case TK_OPTIONS:
 			check_upr("resource options section", "%s:res_options", res->name);
 			EXP('{');
-			options = parse_options(&resource_options_ctx);
+			parse_options(&options, &resource_options_ctx);
 			STAILQ_CONCAT(&res->res_options, &options);
 			break;
 		case TK_CONNECTION:
@@ -1887,12 +1858,7 @@ int was_file_already_seen(char *fn)
 	ENTRY *e;
 	char *real_path;
 
-	e = calloc(1, sizeof *e);
-	if (!e) {
-		log_err("calloc %m\n");
-		exit(E_THINKO);
-	}
-
+	e = checked_calloc(1, sizeof *e);
 	real_path = realpath(fn, NULL);
 	if (!real_path) {
 		real_path = strdup(fn);
